//----------------------------------------------------------------
// main.cpp
//----------------------------------------------------------------
// Copyright of John Wong 1998
//
// You are free to use this source code as long as it is for
// non-commercial use and this copyright notice is retained.
//----------------------------------------------------------------
// Revision History
//
// [JW-MAY.22.1998] v01.00  John Wong
//----------------------------------------------------------------

#define STRICT
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include "main.h"

HINSTANCE	g_hInstance = NULL;
HWND		g_hWndMain = NULL;
int			g_lState = 0;

LRESULT CALLBACK WindowProc(  
	HWND hwnd,		// handle to window
	UINT uMsg,		// message identifier
	WPARAM wParam,	// first message parameter
	LPARAM lParam);	// second message parameter

int WINAPI WinMain(  
	HINSTANCE a_hInstance,		// handle to current instance
	HINSTANCE a_hPrevInstance,	// handle to previous instance
	LPSTR a_lpCmdLine,			// pointer to command line
	int a_nCmdShow)				// show state of window
{
    MSG msg;     
	WNDCLASS wc;     
	
    // register the main window class
    wc.style = 0;         
	wc.lpfnWndProc = (WNDPROC) WindowProc; 
	wc.cbClsExtra = 0;         
	wc.cbWndExtra = 0; 
	wc.hInstance = a_hInstance;         
	wc.hIcon = LoadIcon((HINSTANCE) NULL, IDI_APPLICATION);         
	wc.hCursor = LoadCursor((HINSTANCE) NULL, IDC_ARROW);         
	wc.hbrBackground = (HBRUSH) GetStockObject(WHITE_BRUSH); 
	wc.lpszMenuName =  NULL; 
	wc.lpszClassName = "MainWndClass";          
	if (!RegisterClass(&wc)) 
	{
		return FALSE;     
	}
	
    g_hInstance = a_hInstance;  // save instance handle      
	
	// Create the main window. 
	g_hWndMain = CreateWindow(
		"MainWndClass", 
		"Sample 2", 
        WS_OVERLAPPEDWINDOW, 
		CW_USEDEFAULT, 
		CW_USEDEFAULT, 
        CW_USEDEFAULT, 
		CW_USEDEFAULT, 
		(HWND) NULL, 
        (HMENU) NULL, 
		g_hInstance, 
		(LPVOID) NULL);  
	
    // If the main window cannot be created, terminate     
	// the application.  
    if (!g_hWndMain)         
	{
		return FALSE;
	}
    
	// Show the window and paint its contents.  
    ShowWindow(g_hWndMain, a_nCmdShow);     
	UpdateWindow(g_hWndMain);  
	
    // Start the message loop.      
	while (GetMessage(&msg, (HWND) NULL, 0, 0)) 
    {         
		TranslateMessage(&msg);         
		DispatchMessage(&msg);     
	}  
    
	// Return the exit code to Windows.      
	return msg.wParam; 
}

LRESULT CALLBACK WindowProc(  
	HWND a_hwnd,		// handle to window
	UINT a_uMsg,		// message identifier
	WPARAM a_wParam,	// first message parameter
	LPARAM a_lParam)	// second message parameter
{
	switch (a_uMsg)
	{
	case WM_CHAR:
		{
			int value = a_wParam - '1';	// ascii code in wParam

			if (value >= 0 &&
				value < 4)
			{
				g_lState = value;
				InvalidateRect(a_hwnd, NULL, TRUE);
										// Invalidate the whole client area
			}
		}
		return 0;
	case WM_PAINT:
		switch (g_lState)
		{
		case 0: // text drawing demo
			{
				PAINTSTRUCT ps;
				HDC hDC;

				hDC = BeginPaint(a_hwnd, &ps);

				HFONT hFont;
				HFONT hOldFont;
				LPCSTR str1 = "This is a text drawing demo!";
				LPCSTR str2 = "Text background is opaque.";
				LPCSTR str3 = "Text background is transparent.";
				LPCSTR str4 = "Text is horizontally centered at x-coord 200.";
				LPCSTR str5 = "Text is left aligned at x-coord 200.";
				LPCSTR str6 = "Text is right aligned at x-coord 200.";
				LPCSTR str7 = "DrawText() has a lot more formating\nthan TextOut().\n&Testing ..\n&Testing..";

				TextOut(hDC, 0, 0, str1, strlen(str1));	// use the default for output.

				hFont = CreateFont(
					14,	// height
					0,	// width -- 0 means use the default proportion to the height
					0,
					0,
					FW_NORMAL, // weight of font
					0,	// italic
					0,	// underline
					0,	// strike out
					ANSI_CHARSET, // character set
					OUT_DEFAULT_PRECIS,
					CLIP_DEFAULT_PRECIS,
					DEFAULT_QUALITY,
					VARIABLE_PITCH | FF_SWISS,
					"arial");

				hOldFont = (HFONT) SelectObject(hDC, hFont);

				SetTextColor(hDC, RGB(255, 0, 0));
				SetBkColor(hDC, RGB(0, 255, 0));
				SetTextAlign(hDC, TA_LEFT | TA_TOP);
				SetBkMode(hDC, OPAQUE);

				TextOut(hDC, 0, 20, str2, strlen(str2));

				SetBkMode(hDC, TRANSPARENT);

				TextOut(hDC, 0, 40, str3, strlen(str3));

				SetTextAlign(hDC, TA_CENTER);

				TextOut(hDC, 200, 60, str4, strlen(str4));

				SetTextAlign(hDC, TA_LEFT);

				TextOut(hDC, 200, 80, str5, strlen(str5));

				SetTextAlign(hDC, TA_RIGHT);

				TextOut(hDC, 200, 100, str6, strlen(str6));

				SetTextAlign(hDC, TA_LEFT);

				RECT rc = {0, 120, 400, 240};

				DrawText(
					hDC,
					str7,
					strlen(str7),
					&rc,
					DT_TABSTOP | DT_TOP | DT_LEFT);

				SelectObject(hDC, hOldFont);

				DeleteObject(hFont);

				EndPaint(a_hwnd, &ps);
			}
			break;
		case 1: // line drawing demo
			{
				PAINTSTRUCT ps;
				HDC hDC;

				hDC = BeginPaint(a_hwnd, &ps);

				HPEN hPenRed;
				HPEN hPenGreen;
				HPEN hPenBlue;
				HPEN hOldPen;
				
				hPenRed = CreatePen(PS_SOLID, 5, RGB(255, 0, 0));
				hPenGreen = CreatePen(PS_DASH, 1, RGB(0, 255, 0));
				hPenBlue = CreatePen(PS_DOT, 1, RGB(0, 0, 255));

				hOldPen = (HPEN) SelectObject(hDC, hPenRed);
				
				MoveToEx(hDC, 10, 10, NULL);
				LineTo(hDC, 110, 10);
				LineTo(hDC, 110, 30);
				LineTo(hDC, 10, 30);
				LineTo(hDC, 10, 10);

				SelectObject(hDC, hPenGreen);

				MoveToEx(hDC, 10, 50, NULL);
				LineTo(hDC, 110, 50);
				LineTo(hDC, 110, 70);
				LineTo(hDC, 10, 70);
				LineTo(hDC, 10, 50);

				SelectObject(hDC, hPenBlue);

				MoveToEx(hDC, 10, 90, NULL);
				LineTo(hDC, 110, 90);
				LineTo(hDC, 110, 110);
				LineTo(hDC, 10, 110);
				LineTo(hDC, 10, 90);

				SelectObject(hDC, hOldPen);

				DeleteObject(hPenRed);
				DeleteObject(hPenGreen);
				DeleteObject(hPenBlue);

				EndPaint(a_hwnd, &ps);
			}
			break;
		case 2: // brush sample
			{
				PAINTSTRUCT ps;
				HDC hDC;

				hDC = BeginPaint(a_hwnd, &ps);

				HBRUSH hBrush;

				hBrush = CreateSolidBrush(RGB(0, 128, 0));

				RECT rc = {10, 10, 200, 50};

				FillRect(hDC, &rc, hBrush);

				EndPaint(a_hwnd, &ps);
			}
			break;
		case 3: // bitmap sample
			{
				PAINTSTRUCT ps;
				HDC hDC;

				hDC = BeginPaint(a_hwnd, &ps);

				HBITMAP hBitmap;
				HBITMAP hOldBitmap;
				HDC hDCBitmap;

				hBitmap = (HBITMAP) LoadImage(
					NULL,
					"logo.bmp",
					IMAGE_BITMAP,
					0,
					0,
					LR_LOADFROMFILE);

				hDCBitmap = CreateCompatibleDC(hDC);

				hOldBitmap = (HBITMAP) SelectObject(hDCBitmap, hBitmap);

				BITMAP bm;

				GetObject(hBitmap, sizeof(bm), &bm);

				BitBlt(
					hDC,
					0,
					0,
					bm.bmWidth,
					bm.bmHeight,
					hDCBitmap,
					0,
					0,
					SRCCOPY);

				StretchBlt(
					hDC,
					0,
					bm.bmHeight,
					bm.bmWidth / 2,
					bm.bmHeight / 2,
					hDCBitmap,
					0,
					0,
					bm.bmWidth,
					bm.bmHeight,
					SRCCOPY);

				SelectObject(hDCBitmap, hOldBitmap);

				DeleteObject(hBitmap);
				DeleteDC(hDCBitmap);

				EndPaint(a_hwnd, &ps);
			}
			break;
		}
		return 0;
	case WM_DESTROY:
		PostQuitMessage(0);
		return 0;
	default:
		return DefWindowProc(a_hwnd, a_uMsg, a_wParam, a_lParam);
	}
}

